%% CheckRegular

% Objective: Post-processing regular wave data
% Method: Spectral analyses and finding upcrossings
% Last Modified: 05.2015
% Authors: Loup and Joakim

%   Input
%   9000_output#.bin

function []=CheckRegular()
%%% INPUT 
ramp_period = 10;   %number of periods in the ramp
length_period = 20; %number of period to be run (excluding ramps)
h=20.9; %full scale water depth [m]
steepnesses = {'1/30','7/180','2/45','1/20','1/18'};
S = [1:5];           %which steepnesses to include
lambda=48;  %scaling factor
ph=73.9/lambda;     %model scale pile height [m]
wave_calib=1;       %if checking the wave calibration data, set wave_calib=1. If not, wave_calib can be anything else.

if wave_calib==1
    if h==20.9
        run=346;
    elseif h==30
        run = 321;
    end
else
    if h==20.9
        run=512;
    elseif h==30
        run = 322;
    end 
end
foldername= ['Regular_h' num2str(h)];
data=[];    %data=[S_nom' T_eff' T_nom' (T_eff./T_nom)' H_eff' H_nom' (H_eff./H_nom)']
TF=[];
for jj=S;
steepness = cell2mat(steepnesses(jj));
files = [1:5];
if wave_calib==1
    folderpath = ['/Volumes/MacintoshHD1/RunsOrganized/Wave_calibration/' foldername];
else
    folderpath = ['/Volumes/MacintoshHD1/RunsOrganized/' foldername];
end
stroke_path = [folderpath '/stroke' numToString100(run) '.bin'];

for k = files    
    log_path = [folderpath '/log' numToString100(run) '.txt'];
    fileID = fopen(log_path);
    C = textscan(fileID,'%s %s','HeaderLines',1,'Delimiter',',');
    output_path = [folderpath '/' cell2mat(C{2}(5*(jj-1)+k)) '.bin'];
    
    % determining input data
    input_file = cell2mat(C{1}(5*(jj-1)+k));
    CC = strsplit(input_file,'_');
    A_in = cell2mat(CC(1));
    A_in(end) = [];
    H_in = 2*str2double(A_in);
    T_in = cell2mat(CC(2));
    T_in(end) = [];
    T_in = str2double(T_in);
    
    % reading measured data
    [a,b] = catman_read(output_path);
    t_meas = b(1).data(:);
    x_meas = b(26).data(:);
    ts_meas = b(34).data(:);
    dt = t_meas(2)-t_meas(1);
    M = b(37).data(:);
    M = M - mean(M(1:floor(length(M)/15)));
    accx = b(32).data(:);
    accx = accx-mean(accx);
    
    % removing the ramp
    lowCutoff=floor(find(abs(x_meas)>H_in/700,1)+ramp_period*T_in/sqrt(lambda)/dt);
    highCutoff=floor(lowCutoff+T_in/sqrt(lambda)*0.9*length_period/dt);
    t_meas_cutoff=t_meas(lowCutoff:highCutoff);
    x_meas_cutoff=x_meas(lowCutoff:highCutoff);
    x_meas_cutoff=smooth(x_meas_cutoff);
    accx_cutoff=accx(lowCutoff:highCutoff);
    M_cutoff=M(lowCutoff:highCutoff);
    freq=(1/dt)*linspace(0,1,length(M));
    freq_cutoff=(1/dt)*linspace(0,1,length(M_cutoff));
    
    % determining when upcrossings occur
    counter=1;
    upCrossing=[];
    for ii=1:length(x_meas_cutoff)-1
        if x_meas_cutoff(ii)<0 && x_meas_cutoff(ii+1)>0
            upCrossing(counter)=ii;
            counter=counter+1;
        end
    end
    
    % finding maximum wave elevations and their positions
    findMax=[];
    findMin=[];
    maxPos=[];
    minPos=[];
    counter=1;
    for ii=1:length(upCrossing)-1
        findMax(counter)=max(x_meas_cutoff(upCrossing(ii):upCrossing(ii+1)));
        findMin(counter)=min(x_meas_cutoff(upCrossing(ii):upCrossing(ii+1)));
        maxPos(counter)=find(x_meas_cutoff(upCrossing(ii):upCrossing(ii+1))==findMax(counter),1)*dt+t_meas_cutoff(1)+upCrossing(ii)*dt;
        minPos(counter)=find(x_meas_cutoff(upCrossing(ii):upCrossing(ii+1))==findMin(counter),1)*dt+t_meas_cutoff(1)+upCrossing(ii)*dt;
        counter=counter+1;
    end
    figure;plot(t_meas_cutoff,x_meas_cutoff); hold on; plot(maxPos,findMax,'o');hold on; plot(minPos,findMin,'o');plot_title = ['Regular waves, S = ' steepness ', h = ' num2str(h) ' m, H_{in} = ' num2str(H_in) ' m' ', T_{in} = ' num2str(T_in) ' s'];title(plot_title);
    
    % determining the wave heights and periods based on mean values
    WaveHeights=[];
    Tvec=[];
    for ii=1:length(findMax)-1
        WaveHeights(ii)=abs(findMax(ii))+abs(findMin(ii));
        Tvec(ii)=maxPos(ii+1)-maxPos(ii);
    end
    T_eff(5*(jj-1)+k)=mean(Tvec)*sqrt(lambda);
    H_eff(5*(jj-1)+k)=mean(WaveHeights)*lambda;
    S_nom(5*(jj-1)+k)=str2num(steepness);
    H_nom(5*(jj-1)+k)=H_in;
    T_nom(5*(jj-1)+k)=T_in;
    
    x_meas=x_meas*lambda;
    t_meas=t_meas*sqrt(lambda);
    x_meas_cutoff=x_meas_cutoff*lambda;
    t_meas_cutoff=t_meas_cutoff*sqrt(lambda);
    M = M*lambda^4;
    M_cutoff = M_cutoff*lambda^4;
    freq_cutoff = freq_cutoff/sqrt(lambda);
    
    t_meas_c=t_meas(1:floor(0.6*length(t_meas)));
    x_meas_c=x_meas(1:floor(0.6*length(x_meas)));
    
    % Plotting all the wave elevations and moments with and without cutoff 
    % and saving the figures
    %{
    WelevTimeSeries=floor(rand*1000000);
    figure(WelevTimeSeries);
    %figure;
    plot(t_meas_c,x_meas_c,'b',t_meas_cutoff,x_meas_cutoff,'r')
    leg=legend('Measured wave elevation','Measured wave elevation cut off');
    plot_title = ['Regular waves, S = ' steepness ', h = ' num2str(h) ' m, T_{nom} = ' num2str(T_in) ' s'];
    tit=title(plot_title);
    xl=xlabel('t [s]');
    yl=ylabel('Wave elevation [m]');
    set(xl,'FontSize',16);
    set(yl,'FontSize',16);
    %set(leg,'FontSize',16);
    set(tit,'FontSize',16);
    if wave_calib==1
        filepath = ['/Users/JoakimFrimann-Dahl/Documents/MATLAB/Figures/RegWaves/WelevTimeSeries_wavecalib_h' num2str(floor(h*10)) 'mHeff' num2str(floor(100*H_eff(5*(jj-1)+k))) 'mTeff' num2str(floor(100*T_eff(5*(jj-1)+k))) 's.png'];
    else
        filepath = ['/Users/JoakimFrimann-Dahl/Documents/MATLAB/Figures/RegWaves/WelevTimeSeries_h' num2str(floor(h*10)) 'mHeff' num2str(floor(100*H_eff(5*(jj-1)+k))) 'mTeff' num2str(floor(100*T_eff(5*(jj-1)+k))) 's.png'];
    end
    saveas(WelevTimeSeries,filepath)
   
    MomTimeSeries=floor(rand*1000000);
    figure(MomTimeSeries);
    %figure;
    plot(t_meas,M,'b',t_meas_cutoff,M_cutoff,'r')
    leg=legend('Measured moment','Measured moment cut off');
    plot_title = ['Regular waves, S = ' steepness ', h = ' num2str(h) ' m, T_{nom} = ' num2str(T_in) ' s'];
    tit=title(plot_title);
    xl=xlabel('t [s]');
    yl=ylabel('Moment [Nm]');
    set(xl,'FontSize',16);
    set(yl,'FontSize',16);
    set(leg,'FontSize',16);
    set(tit,'FontSize',16);
    filepath = ['/Users/JoakimFrimann-Dahl/Documents/MATLAB/Figures/RegWaves/MomTimeSeries_h' num2str(floor(h*10)) 'mHeff' floor(num2str(100*H_eff(5*(jj-1)+k))) 'mTeff' floor(num2str(100*T_eff(5*(jj-1)+k))) 's.png'];
    saveas(MomTimeSeries,filepath)
    %}
 
    % finding the period based on a spectral analysis
    t_meas = t_meas - t_meas(1)+5*sqrt(lambda);
    Y = abs(fft(x_meas));
    Y(1) = 0;
    F = (1/((t_meas(2)-t_meas(1))))*linspace(0,1,length(Y));
    m = max(Y);
    I = find(Y == m,1);
    T_meas = 1/(F(I));
    
    
  if wave_calib~=1
    % Finding the transfer function theta [rad]/moment [Nm] for the filtered (DAF removed) values
    if jj==1 && k<=4
        Fc=1/T_meas*1.5;
        [meanFiltMMax,meanFiltThetaMax]=FindingTF_Regular(t_meas_cutoff/sqrt(lambda),dt,freq_cutoff*sqrt(lambda),M_cutoff/lambda^4,accx_cutoff,Fc*sqrt(lambda),T_meas/sqrt(lambda),ph);
        meanFiltMMax_vec(5*(jj-1)+k)=meanFiltMMax;
        meanFiltThetaMax_vec(5*(jj-1)+k)=meanFiltThetaMax;
        TF(5*(jj-1)+k)=meanFiltThetaMax/meanFiltMMax;
    end

    % plotting the moment spectra and saving the results
    %{
    MomSpec=floor(rand*1000000);
    figure(MomSpec);
    plot(freq_cutoff,abs(fft(M_cutoff)),'LineWidth',2);hold on;
    plot([1/T_in 1/T_in],[0,max(abs(fft(M_cutoff)))],'LineWidth',1);
    if h==20.9
        plot([1/3.77 1/3.77],[0,max(abs(fft(M_cutoff)))],'LineWidth',1);
    elseif h==30
        plot([1/3.94 1/3.94],[0,max(abs(fft(M_cutoff)))],'LineWidth',1);
    end
    plot([1/T_meas 1/T_meas],[0,max(abs(fft(M_cutoff)))],'LineWidth',1);
    plot([2/T_meas 2/T_meas],[0,max(abs(fft(M_cutoff)))],'LineWidth',1);
    plot([3/T_meas 3/T_meas],[0,max(abs(fft(M_cutoff)))],'LineWidth',1);
    plot([4/T_meas 4/T_meas],[0,max(abs(fft(M_cutoff)))],'LineWidth',1);
    plot([5/T_meas 5/T_meas],[0,max(abs(fft(M_cutoff)))],'LineWidth',1);
    hold off;
    xlim([0 3.5/sqrt(lambda)])
    graph_title = ['Regular waves, S=' steepness ', h=' num2str(h) ' m, T_{nom}=' num2str(T_in) ' s'];
    tit=title(graph_title);
    leg=legend('Moment spectrum','Input T','Structural eigenfreq','Measured T','2T','3T','4T','5T');
    xl=xlabel('Freq [Hz]');
    set(xl,'FontSize',16);
    set(leg,'FontSize',16);
    set(tit,'FontSize',16);
    filepath = ['/Users/JoakimFrimann-Dahl/Documents/MATLAB/Figures/RegWaves/MomSpec_h' num2str(floor(h*10)) 'mHeff' num2str(floor(100*H_eff(5*(jj-1)+k))) 'mTeff' num2str(floor(100*T_eff(5*(jj-1)+k))) 's.png'];
    saveas(MomSpec,filepath)
    %}
  end
end
end


TF
data=[S_nom' T_eff' T_nom' (T_eff./T_nom)' H_eff' H_nom' (H_eff./H_nom)'];

%autogenerating LaTeX tables
%columnLabels = {'S\textsubscript{nom}' 'T\textsubscript{eff} [s]' 'T\textsubscript{nom} [s]' 'T\textsubscript{eff}/T\textsubscript{nom}' 'H\textsubscript{eff} [m]' 'H\textsubscript{nom} [m]' 'H\textsubscript{eff}/H\textsubscript{nom}'};
%latexpath = ['/Users/JoakimFrimann-Dahl/Desktop/LatexTables/Table_Regular_h' num2str(h)]
%matrix2latexReg(data, latexpath, 'columnLabels', columnLabels, 'alignment', 'c', 'format', '%-6.2f');

% plotting and saving H_eff/H_nom
    Steep=[];
    Sprev=0;
    counter=1;
    HeffHnom=floor(rand*1000000);
    figure(HeffHnom);
    for jj=S
        for k=files
            Steep(jj)=str2num(cell2mat(steepnesses(jj)));
            if Steep(jj)~=Sprev;
            scatter(Steep(jj)*ones(length(S),1),data(counter:(counter+length(S)-1),7));
            hold on
            end
            counter=counter+1;
            Sprev=Steep(jj);
        end
    end
    set(gca,'XTick',Steep);
    format compact
    set(gca,'XTickLabel',steepnesses);
    plot_title = ['Regular waves, h=' num2str(h) ' m'];
    title(plot_title);
    xl=xlabel('Steepnesses');
    yl=ylabel('H_{eff}/H_{nom}');
    set(xl,'FontSize',14);
    set(yl,'FontSize',14);
    filepath = ['/Users/JoakimFrimann-Dahl/Documents/MATLAB/Figures/RegWaves/HeffHnom_h' num2str(floor(h*10)) '.png'];
    saveas(HeffHnom,filepath)
    hold off

% plotting and saving T_eff/T_nom
    Steep=[];
    Sprev=0;
    counter=1;
    TeffTnom=floor(rand*1000000);
    figure(TeffTnom);
    for jj=S
        for k=files
            Steep(jj)=str2num(cell2mat(steepnesses(jj)));
            if Steep(jj)~=Sprev;
            scatter(Steep(jj)*ones(length(S),1),data(counter:(counter+length(S)-1),4));
            hold on
            end
            counter=counter+1;
            Sprev=Steep(jj);
        end
    end
    set(gca,'XTick',Steep);
    format compact
    set(gca,'XTickLabel',steepnesses);
    xl=xlabel('Steepnesses');
    yl=ylabel('T_{eff}/T_{nom}');
    title(plot_title);
    set(xl,'FontSize',14);
    set(yl,'FontSize',14);
    filepath = ['/Users/JoakimFrimann-Dahl/Documents/MATLAB/Figures/RegWaves/TeffTnom_h' num2str(floor(h*10)) '.png'];
    saveas(TeffTnom,filepath)
    hold off
   
end