%% CompareDecay

% Objective: To compare measured decay with the calculated numerical decay
% Method: Solving the rotational equation of motion
% Last Modified: 05.2015
% Authors: Loup and Joakim
%%
function []=CompareDecay()

global I Crot Krot M delta_t tspan
% INPUT PARAMETERS
lambda=48;      %scaling factor
h=20.9/lambda;           %water depth [m]
th = 0.288/lambda;  %pile thickness
rho=1000;       %water density [kg/m^3]
ph=73.9/lambda;          %pile height [m]
rho_p=2700;     %average density of pile [kg/m^3]
m_nac=5.035+pi*(0.245^2-0.135^2)*0.005*rho_p;     %point mass at the top of the pile (nacelle) (including the mass of the flange) [kg]
R=6.912/2/lambda;         %pile radius [m]
t_end=100;      %time duration [s]
CA=1;           %added mass coefficient
Krot=3400;
if h==20.9/lambda           
    C_ratio=0.0140;         %Damping ratio C_ratio=C/C_cr=c/(2*m*omega0)
    fprintf('The experimental damped eigenperiod is %f s\n', 0.54)
    filepath = '/Volumes/MacintoshHD1/RunsOrganized/Decay/9000_decay_test_002.bin';
    lowCutoff_meas=1219;
    highCutoff_meas=3715;
    lowCutoff_num=5217;
    highCutoff_num=7718;
elseif h==30/lambda
    lowCutoff_meas=1091;
    highCutoff_meas=2228;
    lowCutoff_num=5223;
    highCutoff_num=6347;
    C_ratio=0.0249;         %Damping ratio C_ratio=C/C_cr=c/(2*m*omega0)
    fprintf('The experimental damped eigenperiod is %f s\n', 0.57)
    filepath = '/Volumes/MacintoshHD1/RunsOrganized/Decay/9000_decay_test_005.bin';
elseif h==0
    C_ratio=0.0054;
end

% CALCULATING EQ. OF MOTION PARAMETERS
Imp=m_nac*ph^2;                 %Asssuming a point mass (nacelle) on top of the pile [kg*m^2/rad]
I_pile = (1/3)*(rho_p*pi*ph.^3*(R.^2- (R-th).^2) ...
    +rho * pi*h.^3*((R-th).^2+CA*R.^2));
I = I_pile + Imp;
Crot=2*C_ratio*sqrt(I*Krot);    %Assuming linear damping [kg*m^2/(rad*s)]
T0=2*pi*sqrt(I/Krot);           %Undamped eigenperiod [s]
Td=2*pi/sqrt(Krot/I)/sqrt(1-C_ratio^2); %Damped eigenperiod [s]
fprintf('The undamped eigenperiod is %f s\n', T0)
fprintf('The damped eigenperiod is %f s\n', Td)

[a,b] = catman_read(filepath);
t = b(1).data(:);
yData = b(37).data(:); %moment My (x-direction)
delta_t=t(2)-t(1);
fdecay=max(abs(yData))/ph;

time=0:delta_t:t_end;
Fdecay=zeros(1,length(time));
for ii=1:ceil(length(time)/8)
  Fdecay(ii)=fdecay*(ii-1)/ceil(length(time)/8);
end
for ii=ceil(length(time)/8)+1:ceil(2*length(time)/8)
  Fdecay(ii)=fdecay;
end
M=Fdecay*ph;

%Initial conditions [theta_0 (dtheta/dt)_0]: 
y0=[0 0];

%Time span: 
tspan=0:delta_t:t_end;

% Calling ode solver function ode45 
[t,y] = ode45(@eqmotion,tspan,y0);

t_cutoff1=t(lowCutoff_meas:highCutoff_meas)-t(lowCutoff_meas);
yData=yData(lowCutoff_meas:highCutoff_meas);

d2theta=[0; diff(y(:,2))];
Mnum=I*d2theta+Crot*y(:,2)+Krot*y(:,1);
t_cutoff2=t(lowCutoff_num:highCutoff_num)-t(lowCutoff_num);
Mnum=Mnum(lowCutoff_num:highCutoff_num);
figure;
plot(t_cutoff1,yData,t_cutoff2,Mnum)
legend('Measured decay','Numerical decay')
xlabel('t [s]')
ylabel('M_y [Nm]')
plottitle=['Decay test at h = ' num2str(h*lambda) ' m'];
title(plottitle)

end