%%   FilteringButterworth

% Objective: To separate high and low frequency components in a moment time
% series

% Method: Using a 5th-order Butterworth filter

% Last Modified: 05.2015
% Authors: Loup and Joakim

%   Input       Type/Dimension      Description
%   t           vector [s]          time vector
%   M           vector [Nm]         moment time series
%   lambda      double [-]          scaling factor
%   Fc          double [Hz]         cut-off frequency
%%
function []=FilteringButterworth(t,M,lambda,Fc)

freq = 1/(t(2)-t(1))*linspace(0,1,length(t));
N=5; %Order of filtering = 2n

Fs = 200/sqrt(lambda); %Output sampling frequency
Wn=Fc/(Fs/2); %Normalized cut-off frequency = (cutoff frequency)/(Nyquist frequency)
[B_low A_low]=butter(N,Wn,'low'); %Lowpass Butterworth filter
[B_high A_high]=butter(N,Wn,'high'); %Highpass Butterworth filter
M_filtered_low=filtfilt(B_low,A_low,M); %Lowpass moment time series
M_filtered_high=filtfilt(B_high,A_high,M); %Highpass moment time series

%Plotting the separated spectrum
figure;
plot(freq,abs(fft(M_filtered_high)),freq,abs(fft(M_filtered_low)));
xlim([0 0.5])
xlabel('Freq [Hz]')
legend('Moment spectrum, high-pass filter applied','Moment spectrum, low-pass filter applied')

%Plotting the different components
figure;
plot(t,M,t,M_filtered_low,t,M_filtered_high)
xlabel('t [s]');
ylabel('M(t) [Nm]');
legend('Unfiltered','Low-pass filter','High-pass filter') %NOTE: the unfiltered equals the high-filter+low-filter contributions (even if the filter isn't an ideal one)
end