%% FindingTF_Regular

% Objective: Finding the transfer function TF = theta [rad]/M [rad] for the
% lowest-steepness regular waves in order to test the linear response of
% the numerical program

% Method: Filtering away the frequencies above 1.5*f_1, where f_1 is the 
% measured reciprocal to the regular wave frequency

% Last Modified: 05.2015
% Authors: Loup and Joakim

%   Input       Type/Dimension      Description
%   t           vector [s]          time vector
%   dt          double [s]          time step
%   freq        vector [Hz]         frequency vector
%   M           vector [Nm]         moment time series
%   accx        vector [m/s^2]      pile top acceleration
%   Fc          double [Hz]         cut-off frequency
%   T_meas      double [s]          measured regular wave period
%   ph          double [m]          pile height

%   Output      Type/Dimension      Description
%   meanFiltMMax double [Nm]        average filtered moment maxima
%   meanFiltThetaMax double [rad]   average angle maxima
%%
function [meanFiltMMax,meanFiltThetaMax]=FindingTF_Regular(t,dt,freq,M,accx,Fc,T_meas,ph)

% Ideal filtering
M_F=abs(fft(M));
I=find(freq>Fc,1);
M_F(I:end)=0;
M_ideal=real(ifft(M_F));
ACCX=abs(fft(accx));
ACCX(I:end)=0;
accx_ideal=real(ifft(ACCX));
posx=IntegrateAcceleration(t,accx_ideal,0.5/T_meas);
theta=atan(posx/ph);

%Finding the filtered moment upcrossings
counter_M=1;
for ii=1:length(M_ideal)-1
    if M_ideal(ii)<0 && M_ideal(ii+1)>0
        upCrossing_M(counter_M)=ii;
        counter_M=counter_M+1;
    end
end
%Finding the filtered moment maxima and their positions in time
findMax_M=[];
maxPos_M=[];
counter_M=1;
for ii=1:length(upCrossing_M)-1
    findMax_M(counter_M)=max(M_ideal(upCrossing_M(ii):upCrossing_M(ii+1)));
    maxPos_M(counter_M)=find(M_ideal(upCrossing_M(ii):upCrossing_M(ii+1))==findMax_M(counter_M))*dt+t(1)+upCrossing_M(ii)*dt;
    counter_M=counter_M+1;
end

%Finding the pile angle upcrossings
counter_theta=1;
for ii=1:length(theta)-1
    if theta(ii)<0 && theta(ii+1)>0
        upCrossing_theta(counter_theta)=ii;
        counter_theta=counter_theta+1;
    end
end
%Finding the pile angle maxima and their positions in time
findMax_theta=[];
maxPos_theta=[];
counter_theta=1;
for ii=1:length(upCrossing_theta)-1
    findMax_theta(counter_theta)=max(theta(upCrossing_theta(ii):upCrossing_theta(ii+1)));
    maxPos_theta(counter_theta)=find(theta(upCrossing_theta(ii):upCrossing_theta(ii+1))==findMax_theta(counter_theta))*dt+t(1)+upCrossing_theta(ii)*dt;
    counter_theta=counter_theta+1;
end

%Establishing the mean values of the resulting theta and moment time series
meanFiltMMax=mean(findMax_M);
meanFiltThetaMax=mean(findMax_theta);

end