%%   ForcesFNV

% Objective: Calculating the 1st, 2nd and 3rd-order FNV 
% loads on a cylinder 

%Method: Using formulae for irregular waves from [Newman, 1996]

% Last Modified: 03.2015
% Authors: Loup and Joakim

%   Input       Type/Dimension      Description
%   time        vector [m]          time vector
%   h           double [m]          water depth
%   g           double [m/s^2]      gravitational acceleration
%   a           double [m]          cylinder radius
%   wave_elevation vector[m]        wave elevation
%   rho         double [kg/m^3]     water density
%   CD          double [-]          drag coefficient
%   CM          double [-]          inertia coefficient
%   potential_M double [-]          shallow, intermediate or deep selected
%   potential_FNV double [-]        intermediate or deep selected

%   Output      Type/Dimension      Description
%   f1          vector [N]          1st-order load
%   f2          vector [N]          2nd-order load
%   f3          vector [N]          3rd-order load
%%
function [ f1,f2,f3 ] = ForcesFNV( time,h,g,a,wave_elevation,rho,CD,CM,potential_M,potential_FNV)

% finding frequency domain data of the wave elevation
[Y,omega,k,but_a,but_b] = FreqDomain(wave_elevation,time,g,h,potential_FNV);

k(1)=0;
omega(1) = 0.0001;

% vertical step, for integration of first and second-order forces
dz = h/50;
vertVec = -h:dz:0;
kh = k'*h;

% amplitude and phase for each frequency component
WaveCompAmp = abs(Y);
WaveCompPhase = angle(Y);


if (potential_FNV == 2) % deep
  
    Uz = @(z) omega'.*WaveCompAmp.*exp(1i*WaveCompPhase).*exp(k'*z);%
    DUDXz = @(z) -1i*(omega'.^3/g).*WaveCompAmp.*exp(1i*WaveCompPhase).*exp(k'*z);%
    DWDXz = @(z) (omega'.^3/g).*WaveCompAmp.*exp(1i*WaveCompPhase).*exp(k'*z);
    Wz = @(z) 1i*omega'.*WaveCompAmp.*exp(1i*WaveCompPhase).*exp(k'*z);%
    DUDTz = @(z) 1i*omega'.^2.*WaveCompAmp.*exp(1i*WaveCompPhase).*exp(k'*z);%
    DWDTz = @(z) -omega'.^2.*WaveCompAmp.*exp(1i*WaveCompPhase).*exp(k'*z);%
    DUDTDZz = @(z) 1i*(omega'.^4)/g.*WaveCompAmp.*exp(1i*WaveCompPhase).*exp(k'*z);%
    
    
elseif(potential_FNV == 3) % intermediate

    Uz = @(z) g*(k'./omega').*WaveCompAmp.*exp(1i*WaveCompPhase).*cosh(k'*(z+h))./cosh(kh);%actual
    DUDXz = @(z) -1i*g*(k'.^2./omega').*WaveCompAmp.*exp(1i*WaveCompPhase).*cosh(k'*(z+h))./cosh(kh);%actual
    DWDXz = @(z) (k'.^2.*g./omega').*WaveCompAmp.*exp(1i*WaveCompPhase).*sinh(k'*(z+h))./cosh(kh);
    Wz = @(z) 1i*(k'.*g./omega').*WaveCompAmp.*exp(1i*WaveCompPhase).*sinh(k'*(z+h))./cosh(kh);%
    DUDTz = @(z) 1i*k'.*g.*WaveCompAmp.*exp(1i*WaveCompPhase).*cosh(k'*(z+h))./cosh(kh);%
    DWDTz = @(z) -g.*k'.*WaveCompAmp.*exp(1i*WaveCompPhase).*sinh(k'*(z+h))./cosh(kh);%
    DUDTDZz = @(z) 1i*g*k'.^2.*WaveCompAmp.*exp(1i*WaveCompPhase).*sinh(k'*(z+h))./cosh(kh);%

end

TAB_U = cell2mat(arrayfun(Uz,vertVec,'UniformOutput',0));
TAB_U = [TAB_U ; TAB_U(length(TAB_U(:,1)),:) ; conj(TAB_U(length(TAB_U(:,1)):-1:2,:))];
tab_u = real(ifft(TAB_U));

TAB_DUDX = cell2mat(arrayfun(DUDXz,vertVec,'UniformOutput',0));
TAB_DUDX = [TAB_DUDX ; TAB_DUDX(length(TAB_DUDX(:,1)),:) ; conj(TAB_DUDX(length(TAB_DUDX(:,1)):-1:2,:))];
tab_dudx = real(ifft(TAB_DUDX));


% vertical velocity x-derived, z-dependent
TAB_DWDX = cell2mat(arrayfun(DWDXz,vertVec,'UniformOutput',0));
TAB_DWDX = [TAB_DWDX ; TAB_DWDX(length(TAB_DWDX(:,1)),:) ; conj(TAB_DWDX(length(TAB_DWDX(:,1)):-1:2,:))];
tab_dwdx = real(ifft(TAB_DWDX));


% vertical velocity, z-dependent
TAB_W = cell2mat(arrayfun(Wz,vertVec,'UniformOutput',0));
TAB_W = [TAB_W ; TAB_W(length(TAB_W(:,1)),:) ; conj(TAB_W(length(TAB_W(:,1)):-1:2,:))];
tab_w = real(ifft(TAB_W));


% horizontal acceleration, z-dependent
TAB_DUDT = cell2mat(arrayfun(DUDTz,vertVec,'UniformOutput',0));
TAB_DUDT = [TAB_DUDT ; TAB_DUDT(length(TAB_DUDT(:,1)),:) ; conj(TAB_DUDT(length(TAB_DUDT(:,1)):-1:2,:))];
tab_dudt = real(ifft(TAB_DUDT));

% function used in the 2nd-order force calculation (see Newman's paper)
tab_fun = 2.*tab_w.*tab_dwdx + tab_u.*tab_dudx;

z0 = 0;
ETA1 = WaveCompAmp.*exp(1i*WaveCompPhase);
% horizontal velocity frequency components
U = Uz(z0);
% horizontal acceleration frequency components
DUDT = DUDTz(z0);
% horizontal velocity x-derived frequency components
DUDX = DUDXz(z0);
% vertical velocity frequency components
W = Wz(z0); % J
% vertical velocity x-derived frequency components
DWDX = DWDXz(z0); %
% vertical acceleration frequency components
DWDT = DWDTz(z0);
% horizontal acceleration z-derived frequency components
DUDTDZ = DUDTDZz(z0);

% Half of the frequency spectrum was used for calculating the kinematics. 
% Recreating the full spectrum
U=Mirror(U);
DUDT=Mirror(DUDT);
DUDX=Mirror(DUDX);
W=Mirror(W);
DWDX=Mirror(DWDX);
DWDT=Mirror(DWDT);
DUDTDZ=Mirror(DUDTDZ);
ETA1=Mirror(ETA1);

dd = 1;
% going back into time domain
u = filtfilt(but_b,but_a,real(ifft(U)))/dd;
dudt = filtfilt(but_b,but_a,real(ifft(DUDT)))/dd;
dudx = filtfilt(but_b,but_a,real(ifft(DUDX)))/dd;
w = filtfilt(but_b,but_a,real(ifft(W)))/dd;
dwdx = filtfilt(but_b,but_a,real(ifft(DWDX)))/dd;
dwdt = filtfilt(but_b,but_a,real(ifft(DWDT)))/dd;
dudtdz = filtfilt(but_b,but_a,real(ifft(DUDTDZ)))/dd;
eta1 = filtfilt(but_b,but_a,real(ifft(ETA1)))/dd;

% calculating the hydrodynamic loads
[fs,fd,f1] = ForcesMorison(time,h,g,a,wave_elevation,rho,CD,CM,potential_M,0,0,0,0,0 );

f2 = pi*rho*a.^2.*dz.*trapz(tab_fun,2)+ 2*pi*rho*a.^2.*dudt.*eta1;

f31 = pi*rho*a.^2* (eta1.*(...
    dudtdz.*eta1...
    + 2*w.*dwdx...
    + u.*dudx...
    - (2/g)*dudt.*dwdt)...
    -(dudt/g).*(u.^2 + w.^2));
    
f32 = (4*pi*rho*a.^2/g)*(u.^2.*dudt);
f3 = f31 + f32;
end

