%%   ForcesMorison

% Objective: Calculating the hydrodynamic Morison loads on a cylinder 
% Method: Using the Morison equation

% Last Modified: 05.2015
% Authors: Loup and Joakim

%   Input       Type/Dimension      Description
%   time        vector [m]          time vector
%   h           double [m]          water depth
%   g           double [m/s^2]      gravitational acceleration
%   a           double [m]          cylinder radius
%   wave_elevation vector[m]        wave elevation
%   rho         double [kg/m^3]     water density
%   CD          double [-]          drag coefficient
%   CM          double [-]          inertia coefficient
%   potential   double [-]          shallow, intermediate or deep selected
%   wheeler     double [-]          =1 if Wheeler stretching applied
%   Cs_include  double [-]          =1 if slamming component included
%   output_path_cal double [-]      filepath for specific seed
%   alpha       double [-]          parameter for the slam impact velocity
%   impulse_pos double [-]          pos. of impulse relative to upcrossing

%   Output      Type/Dimension      Description
%   FM          vector              Morison inertia force
%   FD          vector              Morison drag force
%   Fs          vector              Morison force with slamming component

%%
function [Fs,FD,FM] = ForcesMorison(time,h,g,a,Wave_elevation,rho,CD,CM,potential,wheeler,include_Cs,output_path_cal,alpha,impulse_pos)

[Y,omega,k] = FreqDomain(Wave_elevation,time,g,h,potential);

k(1)=0;

% vertical step
dz = h/50;
vertVec = -h:dz:0;

if (potential == 1)
    vert_dis = @(z) 1;
elseif (potential == 2)
    vert_dis = @(z) exp(k'*z);
elseif(potential == 3)
    vert_dis = @(z) cosh(k'*(z+h))./cosh(k'*h);    
end

% amplitude and phase for each frequency component
WaveCompHeight = abs(Y);
WaveCompPhase = angle(Y);

% % horizontal velocity, z-dependent
Uz = @(z) k'.*g./(omega').*WaveCompHeight.*exp(1i*WaveCompPhase).*vert_dis(z);
TAB_U = cell2mat(arrayfun(Uz,vertVec,'UniformOutput',0));
TAB_U = [TAB_U ; TAB_U(length(TAB_U(:,1)),:) ; conj(TAB_U(length(TAB_U(:,1)):-1:2,:))];
tab_u = real(ifft(TAB_U));


% horizontal acceleration, z-dependent
DUDTz = @(z) 1i*k'.*g.*WaveCompHeight.*exp(1i*WaveCompPhase).*vert_dis(z);%
TAB_DUDT = cell2mat(arrayfun(DUDTz,vertVec,'UniformOutput',0));
TAB_DUDT = [TAB_DUDT ; TAB_DUDT(length(TAB_DUDT(:,1)),:) ; conj(TAB_DUDT(length(TAB_DUDT(:,1)):-1:2,:))];
tab_dudt = real(ifft(TAB_DUDT));

% calculating the slamming coefficient
if include_Cs == 1
    Cs=slamcoeff(h,g,a,rho,output_path_cal,alpha,impulse_pos);
    if length(Cs) == length(dz.*trapz(abs(tab_u).*tab_u,2))
        Fs=Cs'.*rho*a.*dz.*trapz(abs(tab_u).*tab_u,2);
    else
        Fs=Cs(2:end)'.*rho*a.*dz.*trapz(abs(tab_u).*tab_u,2);
    end
else
    Fs=0;
end

% calculate the Morison hydrodynamic loads
FM=CM*rho*pi*a.^2.*dz.*trapz(tab_dudt,2);
FD=CD*rho*a.*dz.*trapz(abs(tab_u).*tab_u,2);

if(wheeler)
    dz_w =(Wave_elevation(1:length(FM)) + h)/h;
    FM = FM.*dz_w;
    FD = FD.*dz_w;
    Fs = Fs.*dz_w;
end

end

