%%   FreqDomain

% Objective: Finding the Fourier components for the wave elevation and the 
% wave number for a given a circular wave frequency.

% Created: 2015-03-12
% Last Modified: 
% Author: Loup and Joakim

%   Input       Type/Dimension      Description
%   wave_elevation vector [m]       wave elevation time series
%   time        vector [m]          time vector
%   h           double [m]          water depth
%   g           double [m/s^2]      gravitational acceleration
%   potential_  double [-]          shallow, intermediate or deep selected

%   Output      Type/Dimension      Description
%   Y2          vector              Fourier components of the wave elevation
%   W           vector              circular wave frequencies
%   k           vector              wave number based on the cirular freq
%   a           vector              filtering parameter
%   b           vector              filtering parameter

%%
function [Y2,W,k,a,b] = FreqDomain(wave_elevation, time, g, h, potential_)

dt = time(2)-time(1);

% Significant wave height
Hs = 4*std(wave_elevation);
% cut-off frequency as specified in DNV-RP-C205 p41 sec 3.3.2.4
cutoff = sqrt(2*g/Hs)/(2*pi);

% no filtering if the cutoff frq is above the Nyquist frequency
if cutoff>=1/(2*dt)
   X = ['Cutoff frequency is ' num2str(cutoff) 'Hz, above Nyquist frequency'];
   disp(X);
   disp('No filtering was performed');
   wave_elevation_filt = wave_elevation;
   
   
else
% normalized cut-off freq.
cutoff_butter = 2*cutoff*dt;

X = ['Cutoff frequency is ' num2str(cutoff) 'Hz'];
disp(X);
    
% creating the filter
[b,a] = butter(6,cutoff_butter);
    
% filtering
wave_elevation_filt = filtfilt(b,a,wave_elevation);
    
end

% Going into the frequency domain
Y = fft(wave_elevation_filt);
Freq = (1/(dt))*linspace(0,1,length(Y));

% From now on only half the spectrum is used as the second half contains
% the same information. If this was not done, when calculating kinematics
% like acceleration, the second part of the spectrum would be multiplied by
% a non realistic frequency.
Y2 = Y(1:floor(length(Y)/2));
Freq2 = Freq(1:floor(length(Freq)/2));

% Removing mean values
Y2(1) = 0;
Y2(2) = 0;



cutoff_low = Freq2(floor((find(abs(Y2) == max(abs(Y2)),1)/2)));
cutoff_butter_low = 2*cutoff_low*dt;

[b_low,a_low] = butter(6,cutoff_butter_low,'high');
    
% filtering
wave_elevation_filt = filtfilt(b_low,a_low,wave_elevation_filt);

Y = fft(wave_elevation_filt);
Freq = (1/(dt))*linspace(0,1,length(Y));

% From now on only half the spectrum is used as the second half contains
% the same information. If this was not done, when calculating kinematics
% like acceleration, the second part of the spectrum would be multiplied by
% a non realistic frequency.
Y2 = Y(1:floor(length(Y)/2));
Freq2 = Freq(1:floor(length(Freq)/2));


% Transforming the frequency to [rad/s]
W = 2*pi*Freq2;

if (potential_ == 1) %shallow water
    k = W/(sqrt(g*h));
elseif (potential_ == 2) %deep water
    k = W.^2/g;    
elseif(potential_ == 3) %finite water
    k = findk2(W,h);
else
    error('Wrong selection of potential')
end
W(1) = .001;

% Remove mean values
Y2(1) = 0;
Y2(2) = 0;

FreqMax = find(Freq2>2*cutoff,1);

% Components that are considered unimportant are set to 0
Y2(FreqMax+1:end) = 0;

% Wave numbers corresponding to the unimportant frequency components are set
% to a random, low value. Without this, when using the script for very
% deep water, the terms like cosh(k*(h+z)) return 'inf' and are then
% multipled by the frequency component (which in this case is equal to 0).
% Therefore, since these terms will be multipled by 0, we set them to a
% random (finite) value.
k(FreqMax+1:end) = 0.1;

end

