%% GumbelPlot

% Objective: Plotting Gumbel fits, finding the maximum values for the 
% specified quantile

% Method: Using formulae from [Naess and Moan, 2012]
% Last Modified: 05.2015
% Authors: Loup and Joakim

%   Input       Type/Dimension      Description
%   vec_max     double              vector of maxima
%   quantile    double [-]          specified quantile
%   Mean_HsMeas double [m]          mean measured Hs
%   Mean_TpMeas double [s]          mean measured Tp
%   Hs          double [m]          significant wave height
%   Tp          double [s]          peak sepctrum
%   h           double [m]          water depth
%   FM          double [-]          1=force, 2=moment

%   Output      Type/Dimension      Description
%   x           double              max. value (F or M) for spec. quantile
%%
function [x] = GumbelPlot(vec_max,quantile,Mean_HsMeas,Mean_TpMeas,Hs,Tp,h,FM)

vec_max = sort(vec_max);
 s = size(vec_max,1)+1;
    X = 1/s:1/s:1-1/s;
    p = polyfit(vec_max',-log(-log(X)),1);%G = sort(Max(:,k));

    %Gumbel linear regression parameters
    a = 1/p(1);
    b = -p(2)*a;

    %Plotting the probability distribution
    v = 0.85*min(vec_max):min(vec_max)/100:1.15*max(vec_max);
    figF=floor(rand*1000000);
    figure(figF);
    plot(v,exp(-exp(-(v-b)/a)))
    hold on;plot([v(1) v(end)],[quantile quantile])
    hold off;
    xlim([0.86*min(vec_max) 1.14*max(vec_max)]);
    if FM==1
        xlabel('Maximum force [N]');
    elseif FM==2
        xlabel('Maximum moment [Nm]');
    end
    ylabel('Cumulative probability');
    graphtitle=['Probability distribution, mean H_s measured = ' num2str(Mean_HsMeas) ' m, mean T_p measured= ' num2str(Mean_TpMeas) ' s, h = ' num2str(h) ' m'];
    title(graphtitle);
    legend2 = [num2str(quantile) ' quantile'];
    legend('Cumulative probability',legend2,'Location','southeast');
    filepath = ['/Users/JoakimFrimann-Dahl/Documents/MATLAB/Figures/Gumbel/' num2str(FM) 'ProbHs' num2str(floor(Hs*100)) 'Tp' num2str(floor(Tp*100)) 'h' num2str(floor(h*10)) '.png'];
    saveas(figF,filepath)
    
    %max value for specified quantile
    x=b-a*log(log(1/quantile));
    
    %Plotting Gumbel fit
    figGum=floor(rand*1000000);
    figure(figGum);
    plot(vec_max',-log(-log(X)),'r*');
    ax = gca;
    hold on
    plot(ax.XLim,p(1)*ax.XLim+p(2),'b-','LineWidth',2);
    plot([x x],ax.YLim,'--')
    hold off;
    xlim([min(ax.XLim) 1.02*x])
    if FM==1
        xl=xlabel('Maximum force [N]');
    elseif FM==2
        xl=xlabel('Maximum moment [Nm]');
    end
    yl=ylabel('log(log(F))');
    graphtitle=['Gumbel plot, mean H_s measured = ' num2str(Mean_HsMeas) ' m, mean T_p measured= ' num2str(Mean_TpMeas) ' s, h = ' num2str(h) ' m'];
    title(graphtitle);
    legend3 = [num2str(quantile) ' quantile'];
    leg=legend('maxima','Gumbel fit',legend3,'Location','northwest');
    set(xl,'FontSize',18);
    set(yl,'FontSize',18);
    set(leg,'FontSize',18);
    filepath = ['/Users/JoakimFrimann-Dahl/Documents/MATLAB/Figures/Gumbel/' num2str(FM) 'GumbelFitHs' num2str(floor(Hs*100)) 'Tp' num2str(floor(Tp*100)) 'h' num2str(floor(h*10)) '.png'];
    saveas(figGum,filepath)
end