%% PileResponse

% Objective: To calculate the pile response in the time domain, using 
% different excitation force models

% Method: The response is found by solving the equation of motion:
% (Imp+Imd+Ima)*ddtheta(t) + Crot*dtheta(t) + Krot*theta(t) = M(t)

% Last Modified: 05.2015
% Authors: Loup and Joakim
%%

global I Crot Krot M_Morison M_FNV delta_t tspan

savedata = 1;

% INPUT PARAMETERS
lambda=48;      %scaling factor
th = 0.288/lambda;  %pile thickness
rho=1000;       %water density [kg/m^3]
g = 9.81;       %gravitational acceleration [m/s^2]
ph=73.9/lambda;      %pile height [m]
rho_p=2700;     %density of pile [kg/m^3]
m_nac=5.035+pi*(0.245^2-0.135^2)*0.005*rho_p;     %point mass at the top of the pile (nacelle) (including the mass of the flange) [kg]
R=6.912/2/lambda;         %pile radius [m]
CA=1;           %added mass coefficient
CD=0.809;           %drag coefficient
CM=2.0;         %inertia coefficient
Krot=3400;      %assuming linear stiffness [kg*m^2/(rad*s^2)]
damping_multiple = 1; %factor for the damping relative to the decay-measured damping
Ws = 1; %Wheeler stretching: 1 = yes, 0 = no
include_Cs = 0; %include Cs instead of CD: 1 = yes, 0 = no
alpha = 1; %parameter used in the slamming model: u_impact=alpha*C
impulse_pos=1/15;    %specifying when the slam occurs between upcrossings

seastate = 'Irregular_Hs9.04_Tp11.25_h30/';
folderpath = ['/Volumes/MacintoshHD1/RunsOrganized/', seastate];
folderpath_cal = ['/Volumes/MacintoshHD1/RunsOrganized/Wave_calibration/', seastate];

files =  [1:20];

log_M_input_file = cell(1,length(files));
log_FNV_input_file = cell(1,length(files));
log_M_output_model = cell(1,length(files));
log_FNV_output_model = cell(1,length(files));
log_M_calibrated = cell(1,length(files));
log_FNV_calibrated = cell(1,length(files));
log_M_simulated = cell(1,length(files));
log_FNV_simulated = cell(1,length(files));
duration = 3*3600/sqrt(lambda);

log_path = [folderpath 'log.txt'];
log_path_cal = [folderpath_cal 'log.txt'];
fileID = fopen(log_path);
RR=textscan(fileID,'%s %f',1,'HeaderLines',1,'Delimiter','=');
fclose(fileID);
ramp = RR{2}(1);

wave_spectrum = [];
moment_spectrum = [];
acc_meas = [];
acc_calc_M = [];
acc_calc_FNV = [];
  
% Depth characteristics for Morison: 1 = shallow, 2 = deep, 3 = intermediate
potential_M = 3;
% Depth characteristics for FNV: 2 = deep, 3 = intermediate
potential_FNV = 3;

if include_Cs == 0
  if Ws == 1
        if potential_M == 1
            if damping_multiple ~= 2
                model_M = 'Morison_shallow_with_Wheeler';
            else
                model_M = 'Morison_shallow_with_Wheeler_2damping';
            end
        elseif potential_M == 2
            if damping_multiple ~= 2
                model_M = 'Morison_deep_with_Wheeler';
            else
                model_M = 'Morison_deep_with_Wheeler_2damping';
            end
        elseif potential_M == 3
            if damping_multiple ~= 2
                model_M = 'Morison_finite_with_Wheeler';
            else
                model_M = 'Morison_finite_with_Wheeler_2damping';
            end
        end
   elseif Ws == 0
            if potential_M == 1
            if damping_multiple ~= 2
                model_M = 'Morison_shallow';
            else
                model_M = 'Morison_shallow_2damping';
            end
        elseif potential_M == 2
            if damping_multiple ~= 2
                model_M = 'Morison_deep';
            else
                model_M = 'Morison_deep_2damping';
            end
        elseif potential_M == 3
            if damping_multiple ~= 2
                model_M = 'Morison_finite';
            else
                model_M = 'Morison_finite_2damping';
            end
            end
    end
elseif include_Cs == 1
    if Ws == 1
        if potential_M == 1
            if damping_multiple ~= 2
                model_M = 'Morison_slam_shallow_with_Wheeler';
            else
                model_M = 'Morison_slam_shallow_with_Wheeler_2damping';
            end
        elseif potential_M == 2
            if damping_multiple ~= 2
                model_M = 'Morison_slam_deep_with_Wheeler';
            else
                model_M = 'Morison_slam_deep_with_Wheeler_2damping';
            end
        elseif potential_M == 3
            if damping_multiple ~= 2
                model_M = 'Morison_slam_finite_with_Wheeler';
            else
                model_M = 'Morison_slam_finite_with_Wheeler_2damping';
            end
        end
   elseif Ws == 0
            if potential_M == 1
            if damping_multiple ~= 2
                model_M = 'Morison_slam_shallow';
            else
                model_M = 'Morison_slam_shallow_2damping';
            end
        elseif potential_M == 2
            if damping_multiple ~= 2
                model_M = 'Morison_slam_deep';
            else
                model_M = 'Morison_slam_deep_2damping';
            end
        elseif potential_M == 3
            if damping_multiple ~= 2
                model_M = 'Morison_slam_finite';
            else
                model_M = 'Morison_slam_finite_2damping';
            end
            end
    end
end
    
    
if potential_FNV == 2
    if damping_multiple ~= 2
        model_FNV = 'FNV_deep';
    else
        model_FNV = 'FNV_deep_2damping';
    end
elseif potential_FNV == 3
    if damping_multiple ~= 2
        model_FNV = 'FNV_finite';
    else
        model_FNV = 'FNV_finite_2damping';
    end
end



for k = files

    fileID = fopen(log_path);
    fileID_cal = fopen(log_path_cal);
    C = textscan(fileID,'%s %s','HeaderLines',2,'Delimiter',',');
    C_cal = textscan(fileID_cal,'%s %s','HeaderLines',2,'Delimiter',',');
    fclose(fileID);
    fclose(fileID_cal);
    
    output_path = [folderpath cell2mat(C{2}(k)) '.bin'];
    output_names_vec(k) = (C{2}(k));
    
    input_file = cell2mat(C{1}(k));
    CC = strsplit(input_file,'_');
    seed = CC(5);
    seed_vec(k) = str2double(seed);
    
    Hs_spec=cell2mat(CC(3));
    Hs_spec(end) = [];
    Hs_specified(k)=str2double(Hs_spec);
    Tp_spec=cell2mat(CC(4));
    Tp_spec(end) = [];
    Tp_specified(k)=str2double(Tp_spec);
    h_full=cell2mat(CC(6));
    h_full(end-3:end)=[];
    h_full(1)=[];
    h_full=str2double(h_full);
    
    if(h_full==20.9)
        C_ratio = damping_multiple*0.014;
    elseif(h_full==30)
        C_ratio = damping_multiple*0.025;
    end
    
    h=h_full/lambda;
    file_cal = 0;
    for j = files
       if(strcmp(input_file,cell2mat(C_cal{1}(j))))
           input_file_cal = cell2mat(C_cal{1}(j));
           output_path_cal = [folderpath_cal cell2mat(C_cal{2}(j)) '.bin'];
           file_cal = j;
       end
    end
        
    if(~strcmp(input_file,input_file_cal))
        error('Input files being compared are different')
    end
    
    %Reading the calibrated waves, which are used in the excitation force
    %calculations, from file
    [~,b] = catman_read(output_path);
    [~,b_cal] = catman_read(output_path_cal);
    
    W_elev_meas=b_cal(25).data(:);
    t = b_cal(1).data(:);
    t = t-t(1);
    delta_t=t(2)-t(1);


    % CALCULATING EQ. OF MOTION PARAMETERS
    Imp=m_nac*ph^2;   %Assuming a point mass (nacelle) on top of the pile [kg*m^2/rad]
    I_pile = (1/3)*(rho_p*pi*ph.^3*(R.^2- (R-th).^2) ...
        +rho * pi*h.^3*((R-th).^2+CA*R.^2));
    I = I_pile + Imp;
    Crot=2*C_ratio*sqrt(I*Krot);    %Assuming linear damping [kg*m^2/(rad*s)]
    T0=2*pi*sqrt(I/Krot);           %Undamped eigenperiod [s]
    Td=2*pi/sqrt(Krot/I)/sqrt(1-C_ratio^2); %Damped eigenperiod [s]
    fprintf('The undamped eigenperiod is %f s\n', T0)
    fprintf('The damped eigenperiod is %f s\n', Td)

   
    % CALCULATING THE FNV FORCES
     [f1,f2,f3] = ForcesFNV(t,h,g,R,W_elev_meas,rho,CD,CM,potential_M,potential_FNV);
     F_FNV=f1+f2+f3;
     F_FNV=transpose(F_FNV);

     if length(t)~=length(f1)
        figure;plot(t(2:end),f1,t(2:end),f1+f2,t(2:end),f1+f2+f3);leg=legend('f1','f1+f2','f1+f2+f3');xl=xlabel('t [s]');yl=ylabel('Force [N]');set(xl,'FontSize',14);set(yl,'FontSize',14);set(leg,'FontSize',14);graphtitle1=['Specified H_s = ' num2str(Hs_specified(end)) ' m, specified T_p = ' num2str(Tp_specified(end)) ' s, h = ' num2str(h_full) ' m'];title(graphtitle1);
     elseif length(t)==length(f1)
        figure;plot(t,f1,t,f1+f2,t,f1+f2+f3);leg=legend('f1','f1+f2','f1+f2+f3');xl=xlabel('t [s]');yl=ylabel('Force [N]');set(xl,'FontSize',14);set(yl,'FontSize',14);set(leg,'FontSize',14);graphtitle1=['Specified H_s = ' num2str(Hs_specified(end)) ' m, specified T_p = ' num2str(Tp_specified(end)) ' s, h = ' num2str(h_full) ' m'];title(graphtitle1);
     end
    
    % CALCULATING THE MORISON FORCES
    [Fs,FD,FM] = ForcesMorison(t,h,g,R,W_elev_meas,rho,CD,CM,potential_M,Ws,include_Cs,output_path_cal,alpha,impulse_pos);
    if include_Cs == 0
        F_Morison=transpose(FM+FD);
    elseif include_Cs == 1
        F_Morison=transpose(FM+Fs);
        
            [a,b]=catman_read(output_path);
            F=b(36).data(:);
            t_dummy=b(1).data(:);
            t_dummy=t_dummy-t_dummy(1);
            figure;
            subplot(2,1,1);
            plot(t_dummy,F);
            xl=xlabel('t [s]');
            yl=ylabel('Measured shear force [N]');
            set(xl,'FontSize',18);
            set(yl,'FontSize',18);
            subplot(2,1,2);
            plot(t(2:end),F_Morison);
            xl=xlabel('t [s]');
            yl=ylabel('Numerical Morison excitation force [N]'); 
            set(xl,'FontSize',18);
            set(yl,'FontSize',18);
            graphtitle=['Specified H_s = 9.04 m, specified T_p = 11.25 s, h = 30 m, seed 3196'];
            suptitle(graphtitle);
    end
    
    
    %Assuming that the forces act in a point at the MWL
    M_FNV=h*F_FNV;
    M_Morison=h*F_Morison;

    %Response initial conditions [theta_0 (dtheta/dt)_0]: 
    y0=[0 0];

    %Time span: 
    tspan=delta_t:delta_t:length(t)*delta_t;

    % Calling ode solver function ode45 
    [t1,y] = ode45(@eqmotion,tspan,y0);
    [t2,y_FNV] = ode45(@eqmotion_FNV,tspan,y0);
    
    
    W_elev_meas=W_elev_meas-mean(W_elev_meas(1:2000));
    first_step=find(abs(W_elev_meas)>Hs_specified(k)/2500,1);
    F_meas = b(36).data(:);
    F_meas = F_meas-mean(F_meas(1:2000));
    F_meas = CutTime(F_meas,first_step,floor(ramp/delta_t),floor(duration/delta_t));
    M_meas = b(37).data(:);
    M_meas = M_meas-mean(M_meas(1:2000));
    M_meas = CutTime(M_meas,first_step,floor(ramp/delta_t),floor(duration/delta_t));
    accx_meas = CutTime(b(32).data(:),first_step,floor(ramp/delta_t),floor(duration/delta_t));
    W_elev_meas = CutTime(W_elev_meas,first_step,floor(ramp/delta_t),floor(duration/delta_t));

    M_Morison = CutTime(M_Morison,first_step,floor(ramp/delta_t),floor(duration/delta_t));
    M_FNV = CutTime(M_FNV,first_step,floor(ramp/delta_t),floor(duration/delta_t));
    theta_Morison = CutTime(y(:,1),first_step,floor(ramp/delta_t),floor(duration/delta_t));
    theta_FNV = CutTime(y_FNV(:,1),first_step,floor(ramp/delta_t),floor(duration/delta_t));
    f1 = CutTime(f1,first_step,floor(ramp/delta_t),floor(duration/delta_t));
    f2 = CutTime(f2,first_step,floor(ramp/delta_t),floor(duration/delta_t));
    f3 = CutTime(f3,first_step,floor(ramp/delta_t),floor(duration/delta_t));
    FM = CutTime(FM,first_step,floor(ramp/delta_t),floor(duration/delta_t));
    FD = CutTime(FD,first_step,floor(ramp/delta_t),floor(duration/delta_t));
    
    t = (1:length(W_elev_meas))*delta_t;
    freq = (1/delta_t)*linspace(0,1,length(t));
    
    top_pos_M = ph*sin(y(:,1));
    top_acc_M = diff(diff(top_pos_M))/(delta_t).^2;
    top_acc_M = CutTime(top_acc_M,first_step,floor(ramp/delta_t),floor(duration/delta_t));
    top_acc_g_M = sin(y(:,1))*9.81;
    
    
    top_pos_FNV = ph*sin(y_FNV(:,1));
    top_acc_FNV = diff(diff(top_pos_FNV))/(delta_t).^2;
    top_acc_FNV = CutTime(top_acc_FNV,first_step,floor(ramp/delta_t),floor(duration/delta_t));
    top_acc_g_FNV = sin(y_FNV(:,1))*9.81;
    
    acc_meas = [acc_meas,abs(fft(accx_meas))];
    acc_calc_M = [acc_calc_M,abs(fft(top_acc_M))];
    acc_calc_FNV = [acc_calc_FNV,abs(fft(top_acc_FNV))];
    
    if(savedata)
        % saving data to binary files
        mkdir([folderpath,'calc_',model_M]);
        mkdir([folderpath,'calc_',model_FNV]);

        file_name_M = [model_M,'_',num2str(k),'.bin'];
        file_name_FNV = [model_FNV,'_',num2str(k),'.bin'];

        file_path_M_calc = [folderpath, 'calc_',model_M,'/', file_name_M];
        file_path_FNV_calc = [folderpath, 'calc_',model_FNV,'/', file_name_FNV];
        fID_M = fopen(file_path_M_calc,'w');
        fID_FNV = fopen(file_path_FNV_calc,'w');
        M_array = [t',W_elev_meas,FM,FD,zeros(length(t),1),theta_Morison]';
        fwrite(fID_M,M_array,'double');
        FNV_array = [t',W_elev_meas,f1,f2,f3,theta_FNV]';
        fwrite(fID_FNV,FNV_array,'double');

        % writing log with .bin files
        log_M_input_file(k) = cellstr(input_file);
        log_M_output_model(k) = cellstr(C{2}{k});
        log_M_calibrated(k) = cellstr(C_cal{2}{file_cal});
        log_M_simulated(k) = cellstr(file_name_M);

        log_FNV_input_file(k) = cellstr(input_file);
        log_FNV_output_model(k) = cellstr(C{2}{k});
        log_FNV_calibrated(k) = cellstr(C_cal{2}{file_cal});
        log_FNV_simulated(k) = cellstr(file_name_FNV);
    end
    
    
    figure;
    plot(t,accx_meas,t,top_acc_M,t,top_acc_FNV);
    xl=xlabel('t [s]');
    yl=ylabel('accelerations [m/s^2]');
    leg=legend('Measured top acceleration','Top acceleration Morison','Top acceleration FNV');
    set(xl,'FontSize',14);
    set(yl,'FontSize',14);
    set(leg,'FontSize',14);
    
    fclose('all');
end

if(savedata)
    log_M_table = table(log_M_input_file',log_M_output_model',log_M_calibrated',log_M_simulated',...
        'VariableNames',{'Input_File' 'Output_with_model' 'Output_calibration' 'Numerical_model'});
    log_FNV_table = table(log_FNV_input_file',log_FNV_output_model',log_FNV_calibrated',log_FNV_simulated',...
        'VariableNames',{'Input_File' 'Output_with_model' 'Output_calibration' 'Numerical_model'});
    writetable(log_M_table,[folderpath,'calc_',model_M,'/log.txt']); 
    writetable(log_FNV_table,[folderpath,'calc_',model_FNV,'/log.txt']); 
end

mean_acc_meas = smooth(smooth(mean(acc_meas,2)));
mean_acc_calc_M = smooth(smooth(mean(acc_calc_M,2)));
mean_acc_calc_FNV = smooth(smooth(mean(acc_calc_FNV,2)));
figure;
plot(freq,mean_acc_calc_M,freq,mean_acc_calc_FNV,freq,mean_acc_meas);
leg=legend('Morison','FNV','Measured');
xl=xlabel('f [Hz]');
yl=ylabel('acceleration spectra');
set(xl,'FontSize',14);
set(yl,'FontSize',14);
set(leg,'FontSize',14);
graphtitle1=['Seed averaged top-of-pile acceleration spectra for specified H_s = ' num2str(Hs_specified(end)) ' m, specified T_p = ' num2str(Tp_specified(end)) ' s, h = ' num2str(h_full) ' m'];
title(graphtitle1);