%% Realize

% Objective: Generating an irregular wave elevation time series
% Method: Utilizing the procedure specified in [Newland, 1993]
% Last Modified: 02.2015
% Authors: Loup and Joakim

%   Input       Type/Dimension      Description
%   Tmax        double [s]          sea state duration
%   dt          double [s]          time step
%   Hs          double [m]          significant wave height
%   Tp          double [s]          spectral peak period
%   h           double [m]          water depth
%   g           double [m/s^2]      gravitational acceleration
%   seed        double [-]          seed number

%   Output      Type/Dimension      Description
%   t           vector [s]          time vector
%   x           vector [m]          wave elevation time series
%%
function [t,x] = Realize(Tmax,dt,Hs,Tp,gamma, h,g,seed)

%Specifying the time vector
t = 0:dt:Tmax-dt;

%Calculating the wave elevation time series
if mod(length(t),2)==0
    Nfreq = round(length(t)/2);
else
    Nfreq = round(length(t)/2)-1;
    t(end) = [];
end
w=2*pi/(dt*length(t))*(0:Nfreq+1);
dw = 2*pi/Tmax;
WavCompH=zeros(1,Nfreq+2);
Phase=zeros(1,Nfreq+2);
G = JONSWAP_spectrum(w,Hs,Tp,gamma,cutoff);
G(1)=0;
rng(seed);
Phase=rand(1,Nfreq+2)*2*pi;
WavCompH=sqrt(2*G*dw)*Nfreq;
F=WavCompH.*exp(1i*Phase);
F(1)=real(F(1));
F(Nfreq+1)=real(F(Nfreq+1));
FF=[F conj(F(Nfreq-1:-1:2))];
x=real(ifft(FF));


%Checking standard deviation
stdG=sqrt(trapz(w,G))
stdx=std(x)

%Displaying the Hs calculated from the spectrum, to screen
checkhs=4*sqrt(trapz(w,G));
fprintf('The calculated significant wave height [m], Hs=Hm0=4*m0^0.5 = %f \n', checkhs)
end
