%% RegWaveSeries

% Objective: Generating the batch files for regular waves
% Method: Using the wavemaker transfer functions and linear wave theory
% Last Modified: 04.2015
% Authors: Loup and Joakim

%   Output
%   run#.dat
%%
function [] = RegWaveSeries()

%%% INPUT PARAMETERS
gain_factor = 1.03; % to compensate for flap area beeing smaller than channel area
hFS=30;         %full scale water depth [m]
T_vec_buff = [15 12 10 7.5 3.94];   %vector of regular wave periods [s]
S=[1/30 7/180 2/45 1/20 1/18];      %vector of wave steepnesses [-]
A_vec=[];
T_vec=[];
for ii=1:length(S)      %calculating the amplitudes based on the wave steepnesses and periods
    T_vec = [T_vec T_vec_buff];
    for jj=1:length(T_vec_buff)
        A_vec = [A_vec S(ii)*pi/findk(T_vec(jj),hFS)];
    end
end
lambda=48;                       %scaling factor [-]
T_scale=T_vec/sqrt(lambda);        %scaled wave periods [s]
h=hFS/lambda;                    %model scale water depth [m]
A_scale = A_vec/lambda;         % scaled wave amplitude [m]
g = 9.81;                       %gravitational acceleration [m/s^2]
dt=1/50;            %the wavemaker operates with 50 Hz
num_period = 40;    %number of regular periods to be run (including the ramp)
initial_zeros = 5; %time where both trigger and waves are 0 before the simulation (s)
trigger_start = 5; % the time when the trigger is started before the waves
trigger_end = 120; % duration of trigger still on after waves have finished
rest = 8*60; % resting time between 2 runs, in seconds
last_saved_output_str = '9000_511'; %a new folder is created based on the name of the last run

last_saved_output = str2double(last_saved_output_str(6:8));
test = last_saved_output_str(1:4);
new_folder_path = ['/Users/JoakimFrimann-Dahl/Documents/MATLAB/Master_matlab/Data/TestZetaReg/' test '_' numToString100(last_saved_output+1)];
[status,mess,messid] = mkdir(new_folder_path);
if (~isempty(mess))
   error(mess); 
end

input_files = cell(1,length(T_scale));
output_files = cell(1,length(T_scale));
rest_vec = zeros(rest/dt,1); 

total_wave_serie = zeros(initial_zeros/dt,1);
total_s_serie = zeros(initial_zeros/dt,1);
ts = zeros(initial_zeros/dt,1);

total_wave_serie = [total_wave_serie;zeros(trigger_start/dt,1)];
total_s_serie = [total_s_serie;zeros(trigger_start/dt,1)];
ts = [ts;ones(trigger_start/dt,1)];


for i = 1:length(T_scale)
   T = T_scale(i);
   A = A_scale(i);
   omega = 2*pi/T;
   
   t = 0:dt:num_period*T;
   
   % Creating the regular wave elevation
   wave_elev = A*sin(omega*t);
   
   % Applying the transfer functions and the gain factor
   H = biesel(omega,h,0);
   s_amp_b = A/H;
   s_amp = s_amp_b/MechTF(omega);
   s = s_amp*sin(omega*t)*1000*gain_factor;
    
    % Creating ramp
    for q=1:10*T/dt
        s(q)=s(q)*q/(10*T/dt);
        s(end-q+1)=s(end-q+1)*q/(10*T/dt);
    end

    % Creating stroke time series
    total_wave_serie = [total_wave_serie;wave_elev';rest_vec];
    total_s_serie = [total_s_serie;s';rest_vec];
    ts = [ts;ones(length(s),1);ones(trigger_end/dt,1);zeros((rest-trigger_start-trigger_end)/dt,1)];

    if(i~=length(T_scale))
        ts = [ts;ones(trigger_start/dt,1)];
    else
        ts = [ts;zeros(trigger_start/dt,1)];
    end

    
    % For writing the log    
    ID = [num2str(A*lambda) 'm_' num2str(T*sqrt(lambda)) 's_h' num2str(h*lambda)]; 
    input_files(i) = cellstr(ID);
    output_files(i) = cellstr([last_saved_output_str(1:4) '_' numToString100(last_saved_output+i)]);


end

% Plotting the batch
t_tot = dt:dt:length(total_s_serie)*dt;
plot(t_tot,total_s_serie)
hold on;
plot(t_tot,total_wave_serie*1000)
plot(t_tot,ts)
legend('Piston stroke','Wave elevation','Trigger signal')

% Setting up the data matrix
fID_w = fopen([new_folder_path '/wave_elev' numToString100(last_saved_output+1) '.bin'],'w');
fwrite(fID_w,total_wave_serie,'double');

fID_str = fopen([new_folder_path '/stroke' numToString100(last_saved_output+1) '.bin'],'w');
fwrite(fID_str,total_s_serie,'double');

% writing log
log = table(input_files',output_files','VariableNames',{'Input' 'Output'});
log_file_path = [new_folder_path '/log' numToString100(last_saved_output+1) '.txt'];
writetable(log,log_file_path);

data=[total_s_serie ts zeros(1,length(ts))' zeros(1,length(ts))' zeros(1,length(ts))' zeros(1,length(ts))'];
wm_input_file_path = [new_folder_path '/run' numToString100(last_saved_output+1)];
write2mcl(wm_input_file_path,data,dt)

% the maximum piston stroke in the small MARINTEK towing tank is 290 mm
if(max(abs(total_s_serie))>290)
    error('max piston stroke > 290 mm');
end    

fclose('all');
end