%% biesel

% Objective: Find the H/S for a piston-type wavemaker
%            H: wave height for a linear wave [m]
%            S: wave maker stroke
% Method: Using the Biesel transfer function
% Last Modified: 03.2015
% Authors: Trygve Kristiansen (MARINTEK)
% Modified by: Loup and Joakim

%   Input       Type/Dimension      Description
%   omega       double [rad/s]      wave frequency
%   h           double [m]          water depth

%   Output      Type/Dimension      Description
%   HS_pist     double [rad/m]      wave-height-to-stroke transfer function
%%
function [HS_pist]=biesel(omega,h,iwrite)

if omega(1)==0
    omega(1)=omega(2);
end
g     = 9.81;
k     = findk2(omega,h);
kh    = k*h;

HS_pist=[];
HS_pist = 2.*(cosh(2.*kh)-1)./(sinh(2.*kh)+2.*kh);

% It can be proved that phi is monotonically increasing and its limit for 
% large k is 1. Therefore, if for a given k we have phi(k) = k, then for 
% any larger k we have phi(k) = 1.
% Here we find the first value of k for which HS_pist = 2 and we replace all
% values of larger k by 2. This is introduced because Matlab returned NAN
% for too large values of k.
I = find(HS_pist == 2,1);
HS_pist(I:end) = 2;

% Display to screen:
if (iwrite == 1)
  disp(['Piston: H/S = ' num2str(HS_pist)])
end
