%%   slamcoeff

% Objective: Calculating the slamming coefficient to go with the impulse
% force included in the Morison load
% Method: Using the procedure from [Nestegaard, 2004]

% Last Modified: 05.2015
% Authors: Loup and Joakim

%   Input       Type/Dimension      Description
%   h           double [m]          water depth
%   g           double [m/s^2]      gravitational acceleration
%   R           double [m]          cylinder radius
%   rho         double [kg/m^3]     water density
%   output_path_cal double [-]      filepath for specific seed
%   alpha       double [-]          parameter for the slam impact velocity
%   impulse_pos double [-]          pos. of impulse relative to upcrossing

%   Output      Type/Dimension      Description
%   Cs          vector              Slamming coefficient

%%
function [Cs]=slamcoeff(h,g,R,rho,output_path_cal,alpha,impulse_pos)

D=R*2;
[a,b] = catman_read(output_path_cal);

time=b(1).data(:);
time=time-time(1);
dt=time(2)-time(1);
w5=b(23).data(:);
Wave_elevation=b(25).data(:);

figure;
subplot(2,1,1);
plot(time,w5);
xlabel('t [s]');
ylabel('Wave elevation, wave gauge 5 [m]');
subplot(2,1,2);
plot(time,Wave_elevation);
xlabel('t [s]');
ylabel('Wave elevation, wave gauge 7 [m]');

%Select one large event. The phase velocity of this large event is going to
%be applied for all events. Thus, this routine should only be applied to
%analyze individual waves.
peak5time = input('Insert the time of peak occurrence at w5: ');
peak7time = input('Insert the time of peak occurrence at w7: ');

Cphase=2.1/(peak7time-peak5time)    %Assuming a constant phase velocity. The horizontal distance between wave gauges 5 and 7 is 2.1 m. 
Cphase=alpha*Cphase;                %[Nestegaard, 2004] defines the impact velocity as u=alpha*C

%Noting when upcrossing occurs
upx1=[];
counter=1;
for j=1:length(Wave_elevation)-1
    if sign(Wave_elevation(j+1))==1 && sign(Wave_elevation(j))<=0
        upx1(counter)=j+1;
        counter=counter+1;
    end
end

upx=[];
for j=1:length(upx1)-1
    upx(j)=upx1(j)+floor(impulse_pos*(upx1(j+1)-upx1(j)));
end
% Finding local upcrossing periods
Tz = [];
u = [];
for ii=1:length(upx)-1
    Tz(ii)=dt*(upx(ii+1)-upx(ii));
end

%Finding the penetration factor s/D (sD) at each time incidence. As in
%Nestegaard2004, only the parts above MWL follow the 
%Cs=5.15[1/(1+19sD)+0.107sD] equation for the slamming coefficient. The
%rest are set equal to Cs=CD=0.809 (i.e. sD=1)
sD=ones(1,length(Wave_elevation));
for ii=1:length(upx)-1
    counter=0;
    while counter*dt*Cphase<D && sign(Wave_elevation(upx(ii)+counter))==1
        sD(upx(ii)+counter)=counter*dt*Cphase/D;
        counter=counter+1;
    end
end

%The Cs values are calculated using the penetration factors above. A
%vertical wave front is assumed.
for kk=1:length(sD)
    Cs(kk)=5.15*(1/(1+19*sD(kk))+0.107*sD(kk));
end

end